import sys
from antlr4 import *
from .RegoParser import RegoParser
from .RegoParserListener import RegoParserListener

import ast


class RegoListener(RegoParserListener):
    def __init__(self, output, yawl_mode, attribute_matches, verbose):
        self.yawl_mode = yawl_mode # Switch to YAWL mode
        self.attribute_matches = attribute_matches
        self.verbose = verbose

        self.output = output
        self.output.write('import os,sys,inspect\n')
        self.output.write('currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))\n')
        self.output.write('parentdir = os.path.dirname(currentdir)\n')
        self.output.write('sys.path.insert(0,parentdir)\n')
        self.output.write('from PolicyMetagraph import PolicyMetagraph\n\n')
        self.output.write('from mgtoolkit.library import *\n\n\n')
        self.output.write('def generate_metagraph():\n')
        self.output.write('\tpolicy_mg = PolicyMetagraph()\n')

        self.metagraph_name = str(output).split('/')[-1].split('.')[0]
        self.metagraph_filename = "../metagraphs/" + str(output).split('/')[-1].split('.')[0] + ".gv"
        self.rule_count = 0 # Counter for placeholder variables


    # Exit a parse tree produced by RegoParser#module.
    def exitModule(self, ctx:RegoParser.ModuleContext): # Here, we exit the whole tree
        self.output.write('\treturn policy_mg\n')


    # Exit a parse tree produced by RegoParser#g_rule.
    def exitG_rule(self, ctx:RegoParser.G_ruleContext): # Here, we exit the allow block
        g_rule = ctx.getText();
        if self.yawl_mode:
            if not "defaultallow" in g_rule:
                self.output.write('\tpolicy_mg.edges.append(Edge(policy_mg.current_invertex, policy_mg.current_outvertex, attributes=policy_mg.current_propositions))\n')
                self.output.write('\tpolicy_mg.current_invertex = set()\n')
                self.output.write('\tpolicy_mg.current_outvertex = set()\n')
                self.output.write('\tpolicy_mg.current_propositions = set()\n\n')
        else:
            # Add edges and clean
            self.output.write('\tsrc = "rule_' + str(self.rule_count) + '"\n')
            self.output.write('\tpolicy_mg.variables.add(src)\n')
            self.output.write('\tdst = rule_head\n')
            self.output.write('\tpolicy_mg.variables.add(dst)\n')
            self.output.write('\tpolicy_mg.edges.append(Edge({src}, {dst}, attributes=policy_mg.current_propositions))\n')
            self.output.write('\tpolicy_mg.current_propositions = set()\n\n')
            self.rule_count += 1


    # Enter a parse tree produced by RegoParser#rule_head.
    def enterRule_head(self, ctx:RegoParser.Rule_headContext): # Here, we enter the rule name
        if self.yawl_mode:
            pass
        else:
            rule_head = ctx.var().getText()
            self.output.write("\trule_head = '" + rule_head + "'\n")


    # Enter a parse tree produced by RegoParser#literal.
    def enterLiteral(self, ctx:RegoParser.LiteralContext): # Here, we enter a line in rule body
        literal = ctx.getText();
        if self.verbose >= 1:
            print("LITERAL: {}".format(literal))
        if self.yawl_mode:
            if "input==" in literal:
                inputs = ast.literal_eval(literal.split("==")[1])
                if self.verbose >= 2:
                    print(inputs)
                self.output.write('\tinputs = {}\n'.format(inputs))
                # Add inputs to variables_set
                self.output.write('\tfor input in inputs:\n')
                self.output.write('\t\tif input not in policy_mg.variables:\n')
                self.output.write('\t\t\tpolicy_mg.variables.add(input)\n')
                # Add inputs to invertex
                self.output.write('\tpolicy_mg.current_invertex = set(inputs)\n')
            elif "output==" in literal:
                outputs = ast.literal_eval(literal.split("==")[1])
                if self.verbose >= 2:
                    print(outputs)
                self.output.write('\tattribute_matches = {}\n'.format(self.attribute_matches))
                self.output.write('\toutputs = {}\n'.format(outputs))
                # Add inputs to variables_set
                self.output.write('\tfor output in outputs:\n')
                self.output.write('\t\tif any(match in output for match in attribute_matches):\n')
                self.output.write('\t\t\tpolicy_mg.propositions.add(output)\n')
                self.output.write('\t\telse:\n')
                self.output.write('\t\t\tpolicy_mg.variables.add(output)\n')
                # Add inputs to invertex
                self.output.write('\tpolicy_mg.current_outvertex = set(outputs)\n')
            else: # Propositions
                self.output.write('\tpolicy_mg.propositions.add({})\n'.format(literal))
                self.output.write('\tpolicy_mg.current_propositions.add({})\n'.format(literal))

        else:
            literal = literal.replace('"', '\\"') # Sanitize literal
            self.output.write('\tif "' + literal + '" not in policy_mg.propositions:\n')
            self.output.write('\t\tpolicy_mg.propositions.add("' + literal + '")\n')
            self.output.write('\tif "' + literal + '" not in policy_mg.current_propositions:\n')
            self.output.write('\t\tpolicy_mg.current_propositions.add("' + literal + '")\n')
